#+eval=FALSE
rm(list=ls())
graphics.off()

library(zoo)
library(geosphere)
require(rgdal)
library(hydroGOF)
library(hydromad)
require(raster)
# install.packages("hydromad", repos="http://hydromad.catchment.org")

setwd("C:/Users/ingridlr/Documents/GR_SWMM")
#setwd("M:/WD/Student/Projects/Ingrid/GR_SWMM")

load("Precip_all.RDat")                     ## Load precipitation data: all events
load("Runoff_all.RDat")                     ## Load Runoff data: all events
ppdt <- read.table("Data/Precip/P_Events.dat",header=F,skip=2)
source("GRSWMM_functions.R")

## Function for calibrating the GR model (SWMM) 
GRModel <- function(thet) {
  ec <- ifelse(v==0, ev, vs)              # event
  thet <- round(thet, 3)
  write.table(thet, "par.txt", col.names = F, row.names = F)
  
  ## 1. Assign variables: parameter values
  por <- round(thet[1], 3)                         # Soil layers
  FC <- round(thet[2], 3)                          # Soil layers
  Ksat <- round(thet[3], 1)                        # Soil layers
  Kcoeff <- round(thet[4], 1)                      # Soil layers
  Vratio <- round(thet[5], 3)                      # Drainage Mats
  Rough <- round(thet[6], 3)                       # Drainage Mats
  InitSat <- SI_sat[ec]                            #LID_USAGE
  
  ## 2. Edit the input file
  # a) "[LID_CONTROLS]"
  i.LCN <- indexEXT(entry="[LID_CONTROLS]")   
  t.LCN  <- CovTable2(i.L_=i.LCN[3])                 # a.1) SOIL
  t.LCN[[1]][4:5] <- c(por, FC)                      # a.1) SOIL
  t.LCN[[1]][7:8] <- c(Ksat, Kcoeff)                 # a.1) SOIL
  ot.swm[i.LCN[3]] <- FillLine(r=1, t.dat=t.LCN)     # a.1) SOIL
  t.LCN  <- CovTable2(i.L_=i.LCN[4])                 # a.2) DRAINAGE MAT
  t.LCN[[1]][4:5] <- c(Vratio, Rough)                # a.2) DRAINAGE MAT
  ot.swm[i.LCN[4]] <- FillLine(r=1, t.dat=t.LCN)     # a.2) DRAINAGE MAT
  # b) "[LID_USAGE]"
  i.LUS <- indexEXT(entry="[LID_USAGE]")   
  t.LUS  <- CovTable2(i.L_=i.LUS)
  t.LUS[[1]][6] <- InitSat
  ot.swm[i.LUS] <- FillLine(r=1, t.dat=t.LUS)
  # c) "[SUBCATCHMENTS]"
  i.SCA <- indexEXT(entry="[SUBCATCHMENTS]")
  t.SCA <- CovTable2(i.L_=i.SCA)
  t.SCA[[1]][2] <- F_evt[ec]
  ot.swm[[i.SCA[1]]] <- FillLine(r=1, t.dat=t.SCA)
  # d) "[RAINGAGES]"
  i.RNG <- indexEXT(entry="[RAINGAGES]")
  t.RNG <- CovTable2(i.L_=i.RNG)
  t.RNG[[1]][1] <- t.RNG[[1]][7] <- F_evt[ec]
  ot.swm[[i.RNG[1]]] <- FillLine(r=1, t.dat=t.RNG)
  H_rec <- HistRec(Pdat=P_dat[[ec]], Qdat=Q_dat[[s]][[ec]])  ## Historic records
  #(head(P_dat[[ec]]))
  #(head(Q_dat[[s]][[ec]]))
  
  ######################################################################################
  ## Start and End time specifications
  BAnly <- as.POSIXct(paste(H_rec[[3]][1], H_rec[[3]][2]), format="%m/%d/%Y %H:%M:%S", tz="UTC")
  BAnly <- BAnly-(1*60*60)
  BAnly <- as.character(c(format(BAnly, format="%m/%d/%Y"), format(BAnly, format="%H:%M:%S")))
  #print(BAnly)
  ot.swm[15] <- replaceINP(Ln=15, new_dta=BAnly[1]) #change from 3 to 4 if want to start reporting from runoff
  ot.swm[16] <- replaceINP(Ln=16, new_dta=BAnly[2])
  ot.swm[17] <- replaceINP(Ln=17, new_dta=unlist(strsplit(H_rec[[4]][1]," "))[1])
  ot.swm[18] <- replaceINP(Ln=18, new_dta=unlist(strsplit(H_rec[[4]][1]," "))[2])
  ot.swm[19] <- replaceINP(Ln=19, new_dta=unlist(strsplit(H_rec[[4]][length(H_rec[[4]])]," "))[1])
  dmy <- unlist(strsplit(unlist(strsplit(H_rec[[4]][length(H_rec[[4]])]," "))[2],":"))
  ot.swm[20] <- replaceINP(Ln=20, new_dta=paste(dmy[1],as.character(I(as.numeric(dmy[2])+1)),dmy[3], sep=":"))
  ######################################################################################
  writeLines(ot.swm, con=paste("Temp/L34b_", stn,".inp", sep=""))
  
  ## 3. Run the SWMM model: excute the .bat file
  system(paste("swmm_",stn,".bat", sep=""))
  
  ## 4. Evaluation. Print to Files. Figures.
  return(EvalPlot(vrf=v, plt=pl, thet=thet, Hrec=H_rec))
}

F_evt <- paste("Event_", 1:12, sep="")      ## List: names of precipitation event files
stns <- paste("GR",1:2,sep="")
n_stns <- paste(stns,"_out",sep="")
#SI_sat <- c(18,32,27,32,28,27,26,32,22,31,19,24) #GR2 SWMM corrections
SI_sat <- c(22,35,35,35,26,26,24,35,27,35,14,24) #GR1 SWMM corrections

s <- 2
#for(s in 1:length(stns)) {
  stn <- stns[s]
  for(ev in c(1:6,8,9,12)) {
    pl <- v <- 0
    f_name <- paste("Cal_L34b_",stn,"_model_6P_Event_",ev,".txt",sep="")
    n_stn <- n_stns[s]                     ## Node corresponding to gauging station
    ot.swm <- readLines(con=paste("InpFiles/L34b_", stn,".inp", sep=""))                                        ## Read the reference input file
    
    ## Initial, lower and upper parameter values - disse m jeg endre
    p.int <- c(   0.65,    0.40,     2000,   40,    0.72,   0.2) #p fc cond conds vf n
    p.lwr <- c(   0.55,    0.20,       36,    5,    0.05,  0.01)
    p.upr <- c(   0.75,    0.54,     4200,   60,       1,   0.4)

    write.table(rbind(c("NSE","SSR","%Pdf", "d_cnY", "d_cnX", "T_lag", paste("P", 1:length(p.int), sep=""))), f_name,
                sep="\t", quote=F, append=F, col.names=F, row.names=F)
    m.opt <- SCEoptim(FUN=GRModel, par=p.int, lower=p.lwr, upper=p.upr, control=list(fnscale=-1))
    pl <- 1
    o.par <- m.opt$par
    # aaa__ <- as.numeric(unlist(read.table("par.txt", header=F)))
    # o.par <- thet <- aaa__
    P.sim <- GRModel(thet=o.par)
    ot.swm <- readLines(con=paste("Temp/L34b_", stn,".inp", sep=""))
    writeLines(ot.swm, con=paste("InpFiles/L34b_", stn, F_evt[ev],".inp", sep=""))
    
    ### Verification step
    f_nam2 <- paste("Verification_L34b_",stn,"_model_6P_","CalSet_", F_evt[ev],".txt",sep="")
    write.table(rbind(c("Event","NSE","SSR","%Pdf", "d_cnY", "d_cnX", "T_lag")), f_nam2, sep="\t", quote=F, append=F, col.names=F, row.names=F)
    for(vs in c(2:4,6:9,12)) {
      v <- 1
      ot.swm <- readLines(con=paste("InpFiles/L34b_", stn,".inp", sep=""))     ## Read the reference input file
      P.sim <- GRModel(thet=o.par)
      ot.swm <- readLines(con=paste("Temp/L34b_", stn,".inp", sep=""))
      writeLines(ot.swm, con=paste("InpFiles/L34b_", stn, F_evt[ev],"_VerSet_", F_evt[vs],".inp", sep=""))
    }  
  }

  