## Function for converting a set of lines into a table
CovTable <- function(it_l, rf) {
  i.spr <- unlist(strsplit(ot.swm[-rf+it_l[1]], ""))
  p.spr <- subset(1:length(i.spr), i.spr==" ")
  p.be <- rbind(c(1, 1+p.spr), c(p.spr, length(i.spr)))
  d.df <- matrix(" ", ncol=ncol(p.be), nrow=length(it_l))
  for(j in 1:nrow(d.df)) {
    dmmm <- unlist(strsplit(ot.swm[it_l[j]],""))
    for(v in 1:ncol(d.df)) {
      dmm2 <- dmmm[p.be[1,v]:p.be[2,v]]
      v.va <- dmm2[dmm2!=" "]
      if(all(!is.na(v.va)))
        d.df[j,v] <- noquote(paste(v.va, sep="", collapse=""))
    }
  }
  return(list(d.df, p.be))
}
## Function for converting a single line in to a table
CovTable2 <- function(i.L_) {
  dummy <- dumy2 <- CovTable(it_l=i.L_, rf=0)
  df_dm <- diff(dummy[[2]])
  dumy2[[1]] <- dumy2[[1]][,df_dm>0]
  dumy2[[2]] <- dumy2[[2]][,df_dm>0]
  dumy2[[2]][2,1:I(-1+ncol(dumy2[[2]]))] <- -1+dumy2[[2]][1,2:ncol(dumy2[[2]])]
  dumy2[[2]][2,ncol(dumy2[[2]])] <- max(dummy[[2]])
  return(dumy2)
}
## Function for extracting indexes for different entries
indexEXT <- function(entry) {
  i.dummy <- match(entry, ot.swm)
  l.blnky <- subset(1:length(ot.swm), ot.swm=="")
  i.dummy <- c(I(3+i.dummy):I(-1+l.blnky[l.blnky>i.dummy][1]))
  return(i.dummy)
}
## Function for directly replacing input values directly (line by line)
replaceINP <- function(Ln, new_dta) {
  d_xx <- strsplit(as.character(new_dta),"")
  d_vv <- strsplit(as.character(ot.swm[Ln]),"")
  d_vv[[1]][I(-(length(d_xx[[1]]))+length(d_vv[[1]])+1):length(d_vv[[1]])] <- d_xx[[1]]
  return(paste(d_vv[[1]], collapse =""))
}
## Function for converting tables into .inp file format
FillLine <- function(r, t.dat) {
  dummy2 <- rep(" ", t.dat[[2]][nrow(t.dat[[2]]), ncol(t.dat[[2]])])
  if(is.table(t.dat[[1]])) f.inf <- t.dat[[1]][r,]
  else f.inf <- t.dat[[1]]
  for(i.tm in 1:length(f.inf)) {
    i.rpm <- unlist(strsplit(f.inf[i.tm], ""))
    if(length(i.rpm)!=0) {
      l.rpm <- min(length(i.rpm), length(t.dat[[2]][1,i.tm]:t.dat[[2]][2,i.tm]))
      dummy2[t.dat[[2]][1,i.tm]:I(t.dat[[2]][1,i.tm]+l.rpm-1)] <- i.rpm[1:l.rpm]
    }
    if(length(i.rpm)==0)
      dummy2[t.dat[[2]][1,i.tm]:t.dat[[2]][2,i.tm]] <- rep(f.inf[i.tm],length(t.dat[[2]][1,i.tm]:t.dat[[2]][2,i.tm]))
  }
  dummy2 <- paste(dummy2, sep=" ", collapse="")
  return(dummy2)
}
## Historic records
HistRec <- function(Pdat, Qdat) {
  t_prp <- c(as.character(Pdat[1,1]),as.character(Pdat[1,2]))
  t_rec <- format(Qdat$Date, format="%m/%d/%Y %H:%M:%S", tz="UTC")
  i_pkQ <- subset(1:length(Qdat$Q),Qdat$Q==max(Qdat$Q))
  i_00Q <- subset(1:length(Qdat$Q),Qdat$Q==0)
  i_rng <- i_00Q[i_00Q>i_pkQ[1]][1]
  Q_evl <- Qdat$Q[1:i_rng]
  t_evl <- t_rec[1:i_rng]
  return(list(t_evl,Q_evl,t_prp,t_rec))
}
## Function for evauating model performance, writing and ploting results
EvalPlot <- function(vrf,plt,thet,Hrec) {
  t_evl <- Hrec[[1]]
  Q_evl <- Hrec[[2]]
  
  ## Function for evauating model performance
  EvalFUN <- function(t.sim, t.evl, Q.evl, s.sim) {
    e.NSE <- round(NSE(s.sim, Q.evl),3)
    e.SSR <- round(ssq(s.sim, Q.evl),3)
    d.cnY <- round((mean(Q.evl)-mean(s.sim)),5)
    p.Pdf <- round(100*abs(max(Q.evl)-max(s.sim))/max(Q.evl),1)
    i_pkO <- subset(1:length(Q.evl), Q.evl==max(Q.evl))
    i_pkS <- subset(1:length(s.sim), s.sim==max(s.sim))
    if(max(s.sim[c.set])!=0) {
      o.cnX <- t_sim[floor(sum((1:length(Q.evl))*(Q.evl/sum(Q.evl))))]
      s.cnX <- t_sim[floor(sum((1:length(s.sim))*(s.sim/sum(s.sim))))]
      d.cnX <- as.numeric(difftime(o.cnX,s.cnX, units="mins"))
      i_pkO <- t_sim[i_pkO[floor(runif(1,1,length(i_pkO)))]]
      i_pkS <- t_sim[i_pkS[floor(runif(1,1,length(i_pkS)))]]
      T_lag <- as.numeric(difftime(i_pkO,i_pkS, units="mins"))
    }
    else T_lag <- d.cnX <- Inf
    return(list(e.NSE, e.SSR, p.Pdf, d.cnY, d.cnX, T_lag))
  }
  ## Function for generating PQ series
  PQSeries <- function(ev_N, t.evl, Q.evl, s.sim) {
    p_dfr <- data.frame(Date=t.evl,P=0, Qo=Q.evl, Qs=0)
    P.datN <- as.POSIXct(paste(P_dat[[ev_N]][,1],P_dat[[ev_N]][,2]), format="%m/%d/%Y %H:%M:%S", tz="UTC")
    t_i22 <- match(P.datN, p_dfr$Date)
    d.pp <- ppdt$V7[ppdt$V1==F_evt[ev_N]]
    p_dfr$P[t_i22[!is.na(t_i22)]] <- d.pp[subset(1:length(t_i22), !is.na(t_i22))]
    p_dfr$Qs <- s_sim[c.set]
    p_dfr <- p_dfr[order(p_dfr$Date),]
    p_dfr <- zoo(p_dfr[,-1], p_dfr$Date)
    return(p_dfr)
  }
  
  m.out <- readLines(paste("Temp/L34b_", stn,".rpt", sep=""), warn=F) #Change this from BRC/GR
  c.end <- subset(1:length(m.out), m.out=="  ")
  b.sim <- as.numeric(5+match(paste("  <<< Node ", n_stn, " >>>", sep=""), m.out))
  e.sim <- as.numeric(I(-1+(c.end[c.end>b.sim][1])))
  m.out <- noquote(m.out[b.sim:e.sim])
  t_sim <- unlist(lapply(m.out, substr, 3, 22))
  s_sim <- as.numeric(unlist(lapply(m.out, substr, 29, 33)))
  t_sim <- as.POSIXct(t_sim, format="%b-%d-%Y %H:%M:%S", tz="UTC")
  t_evl <- as.POSIXct(t_evl,  format="%m/%d/%Y %H:%M:%S", tz="UTC")
  c.set <- match(t_evl, t_sim)     ##  observation and simulation time steps that match
  t_evl <- t_evl[subset(1:length(t_evl),!is.na(c.set))]
  Q_evl <- Q_evl[subset(1:length(Q_evl),!is.na(c.set))]                            # For evaluation
  c.set <- c.set[!is.na(c.set)]
  
  p_evl <- EvalFUN(t.sim=t_sim[c.set], t.evl=t_evl, Q.evl=Q_evl, s.sim=s_sim[c.set])
  out <- p_evl[[1]]
  
  ## 5. Write evaluation metrices and parameters to a file
  if(vrf==0) {
    w.fle <- rbind(c(unlist(p_evl), thet))
    PQSr <- PQSeries(ev_N=ev, t.evl=t_evl, Q.evl=Q_evl, s.sim=s_sim[c.set])
    PQSr <- data.frame(Date=index(PQSr), PQSr)
    write.table(w.fle, f_name, sep="\t", quote=F, append=T, col.names=F, row.names=F)
    write.table(PQSr, paste("Final_PQoQs_",stn,"_", F_evt[ev],".txt",sep=""), sep="\t", row.names=F, quote=F)
    #save(file=paste("Final_PQoQs_",stn,"_", F_evt[ev],".RDat",sep=""), PQSr)
  }
  if(vrf==1) write.table(cbind(F_evt[vs],rbind(unlist(p_evl))), f_nam2, sep="\t", quote=F, append=T, col.names=F, row.names=F)
  
  ## 6. Graphic illustrations
  if(plt==1) {
    if(vrf==0) {
      PQSr <- PQSeries(ev_N=ev, t.evl=t_evl, Q.evl=Q_evl, s.sim=s_sim[c.set])
      pMa1 <- paste("Green Roof ",s,"  ","Event #", ev, sep="")
      pfnm <- paste("HydroHyetograph_",stn,"_", F_evt[ev],".png",sep="")
      ltxt <- c(paste(stn," (", F_evt[ev],")", sep=""), "Obs", "Sim") 
    }
    if(vrf==1) {
      PQSr <- PQSeries(ev_N=vs, t.evl=t_evl, Q.evl=Q_evl, s.sim=s_sim[c.set])
      pMa1 <- paste("Green Roof ",s,"  ","Event #", ev," (VerSet:","Event #",vs,")", sep="")
      pfnm <- paste("HydroHyetograph_",stn,"_", F_evt[ev],"_VerSet_", F_evt[vs],".png",sep="")
      ltxt <- c(paste(stn," (", F_evt[vs],")", sep=""), "Obs", "Sim") 
    }
    
    Q_max <- 0.2
    P_max <- 2.5
    
    plot(PQSr[,2], ylim=c(0,Q_max), type='l', main=pMa1, ylab="Runoff (LPS)", xlab=as.character(format(index(PQSr[3]), format="%d %B %Y")))
    lines(PQSr[,3], col=2)
    legend("topright", legend=c("NSE","SSR",p_evl[[1]],p_evl[[2]]), bg="white", ncol=2, bty='n')
    legend("right", legend=c("Obs","Sim"), bg="white", ncol=1, bty='n', col=1:2, text.col=1:2, lty=c(1,1))
    
    y2 <- (0.7*Q_max)+(0.3*P_max)
    t_lab <- format(index(PQSr), format="%d%b %H:%M", tz="UTC")
    t_lab <- t_lab[match(seq(index(PQSr)[1],index(PQSr)[nrow(PQSr)],I(1*60*60)),index(PQSr))]
    
    png(filename=pfnm, width=800, height=600)
    par(mar=c(4.5,2.4,1,1.5), mgp=c(2,0.5,0))
    plot((0.5*y2*(PQSr[,2])/Q_max), ylim=c(0,y2), type='l', col=4, main="", xaxt='n', yaxt='n', ylab="",xlab="", yaxs="i")
    lines((0.5*y2*(PQSr[,3])/Q_max), col=2)
    axis(1, at=seq(index(PQSr)[1],index(PQSr)[nrow(PQSr)],I(1*60*60)), labels=t_lab, las=2, tcl=-0.3, cex.axis=0.8)
    axis(2, at=(0.5*y2*seq(0,(Q_max),0.1)/Q_max),labels=seq(0,(Q_max),0.1), las=0, cex.axis=0.8)
    axis(2, at=(0.5*y2*seq(0,(Q_max),0.05)/Q_max),labels=F, tcl=-0.4)
    axis(2, at=(0.5*y2*seq(0,(Q_max),0.01)/Q_max),labels=F, tcl=-0.3)
    mtext("Runoff (LPS)", 2, line=1.5, cex=0.9)
    par(new=TRUE)
    plot(0.5*PQSr[,1]*y2/P_max, ylim=c(y2,0), type='h', main="", xaxt='n', yaxt='n', ylab="",xlab="", yaxs="i")
    axis(4, at=rev(0.5*y2*seq(0,P_max,0.5)/P_max), labels=rev(seq(0,P_max,0.5)), las=0, cex.axis=0.8)
    axis(4, at=rev(0.5*y2*seq(0,P_max,0.1)/P_max), labels=F, tcl=-0.3)
    mtext("Precipitation (mm)", 4, line=0, cex=0.9)
    box()
    legend("topright", legend=ltxt, ncol=1, bty='n', col=c(1,4,2), text.col=c(1,4,2), lty=c(0,1,1))
    legend("right", legend=c("NSE","SSR",p_evl[[1]],p_evl[[2]]), ncol=2, bty='n')
    dev.off()
  }
 return(out)
}
